@@ -35,6 +35,7 @@ from tempfile import mkstemp
 import paramiko
 from paramiko.py3compat import PY2, b, u, StringIO
 from paramiko.common import o777, o600, o666, o644
+from tests import skipUnlessBuiltin
 from tests.stub_sftp import StubServer, StubSFTPServer
 from tests.loop import LoopSocket
 from tests.util import test_path
@@ -276,6 +277,39 @@ class SFTPTest (unittest.TestCase):
             except:
                 pass
 
+
+    def test_5a_posix_rename(self):
+        """Test posix-rename@openssh.com protocol extension."""
+        try:
+            # first check that the normal rename works as specified
+            with sftp.open(FOLDER + '/a', 'w') as f:
+                f.write('one')
+            sftp.rename(FOLDER + '/a', FOLDER + '/b')
+            with sftp.open(FOLDER + '/a', 'w') as f:
+                f.write('two')
+            try:
+                sftp.rename(FOLDER + '/a', FOLDER + '/b')
+                self.assertTrue(False, 'no exception when rename-ing onto existing file')
+            except (OSError, IOError):
+                pass
+
+            # now check with the posix_rename
+            sftp.posix_rename(FOLDER + '/a', FOLDER + '/b')
+            with sftp.open(FOLDER + '/b', 'r') as f:
+                data = u(f.read())
+            self.assertEqual('two', data, "Contents of renamed file not the same as original file")
+
+        finally:
+            try:
+                sftp.remove(FOLDER + '/a')
+            except:
+                pass
+            try:
+                sftp.remove(FOLDER + '/b')
+            except:
+                pass
+
+
     def test_6_folder(self):
         """
         create a temporary folder, verify that we can create a file in it, then
@@ -817,6 +851,35 @@ class SFTPTest (unittest.TestCase):
         sftp_attributes = SFTPAttributes()
         self.assertEqual(str(sftp_attributes), "?---------   1 0        0               0 (unknown date) ?")
 
+    @skipUnlessBuiltin('buffer')
+    def test_write_buffer(self):
+        """Test write() using a buffer instance."""
+        data = 3 * b'A potentially large block of data to chunk up.\n'
+        try:
+            with sftp.open('%s/write_buffer' % FOLDER, 'wb') as f:
+                for offset in range(0, len(data), 8):
+                    f.write(buffer(data, offset, 8))
+
+            with sftp.open('%s/write_buffer' % FOLDER, 'rb') as f:
+                self.assertEqual(f.read(), data)
+        finally:
+            sftp.remove('%s/write_buffer' % FOLDER)
+
+    @skipUnlessBuiltin('memoryview')
+    def test_write_memoryview(self):
+        """Test write() using a memoryview instance."""
+        data = 3 * b'A potentially large block of data to chunk up.\n'
+        try:
+            with sftp.open('%s/write_memoryview' % FOLDER, 'wb') as f:
+                view = memoryview(data)
+                for offset in range(0, len(data), 8):
+                    f.write(view[offset:offset+8])
+
+            with sftp.open('%s/write_memoryview' % FOLDER, 'rb') as f:
+                self.assertEqual(f.read(), data)
+        finally:
+            sftp.remove('%s/write_memoryview' % FOLDER)
+
 
 if __name__ == '__main__':
     SFTPTest.init_loopback()
